;******************************************************************************
;   This file is a basic template for assembly code for a PIC18F2431. Copy    *
;   this file into your project directory and modify or add to it as needed.  *
;                                                                             *
;   Refer to the MPASM User's Guide for additional information on the         *
;   features of the assembler.                                                *
;                                                                             *
;   Refer to the PIC18F2331/2431/4331/4431 Data Sheet for additional          *
;   information on the architecture and instruction set.                      *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Filename:                                                                *
;    Date:                                                                    *
;    File Version:                                                            *
;                                                                             *
;    Author:                                                                  *
;    Company:                                                                 *
;                                                                             * 
;******************************************************************************
;                                                                             *
;    Files Required: P18F2431.INC                                             *
;                                                                             *
;******************************************************************************

	LIST P=18F2331		;directive to define processor
	#include <P18F2331.INC>	;processor specific variable definitions
;	LIST P=18F2420		;directive to define processor
;	#include <P18F2420.INC>	;processor specific variable definitions

;******************************************************************************
;Configuration bits

    CONFIG	OSC = HS             ;HS oscillator
    CONFIG	BOREN = ON           ;Brown-out Reset enabled
	CONFIG	BORV = 42            ;VBOR set to 4.2V could be 27 = 2.7V
	CONFIG	PWRTEN = ON          ;PWRT enabled
	CONFIG	WDTEN = OFF           ;WDT enabled
	CONFIG	WDPS = 256           ;1:256
	CONFIG	MCLRE = OFF          ;Disabled
	CONFIG	STVREN = ON          ;Stack full/underflow will cause Reset
	CONFIG	LVP = OFF            ;Low-voltage ICSP disabled !!!!!! NECESSARY !!!!!!!

#define		Project		"SR_Motor_3_Phase
#define		Version		"0.10"
;Port B
#define		DRVA1	RB0
#define		DRVA2	RB1
#define		DRVB1	RB2
#define		DRVB2	RB3
#define		DRVC1	RB4
#define		DRVC2	RB5


;******************************************************************************
;Variable definitions
; These variables are only needed if low priority interrupts are used. 
; More variables may be needed to store other special function registers used
; in the interrupt routines.

		CBLOCK	0x080
		WREG_TEMP	;variable used for context saving 
		STATUS_TEMP	;variable used for context saving
		BSR_TEMP	;variable used for context saving
		ENDC

		CBLOCK	0x000
		EXAMPLE		;example of a variable in access RAM
		ENDC

;******************************************************************************
;EEPROM data
; Data to be programmed into the Data EEPROM is defined here

		ORG	0xf00000

		DE	"SR_Motor_3_Phase", 0,1,2,3,4,5

;******************************************************************************
;Reset vector
; This code will start executing when a reset occurs.

		ORG	0x0000

		goto	Main		;go to start of main code

;******************************************************************************
;High priority interrupt vector
; This code will start executing when a high priority interrupt occurs or
; when any interrupt occurs if interrupt priorities are not enabled.

		ORG	0x0008

		bra	HighInt		;go to high priority interrupt routine

;******************************************************************************
;Low priority interrupt vector and routine
; This code will start executing when a low priority interrupt occurs.
; This code can be removed if low priority interrupts are not used.

		ORG	0x0018

		movff	STATUS,STATUS_TEMP	;save STATUS register
		movff	WREG,WREG_TEMP		;save working register
		movff	BSR,BSR_TEMP			;save BSR register

;	*** low priority interrupt code goes here ***


		movff	BSR_TEMP,BSR			;restore BSR register
		movff	WREG_TEMP,WREG		;restore working register
		movff	STATUS_TEMP,STATUS	;restore STATUS register
		retfie

;******************************************************************************
;High priority interrupt routine
; The high priority interrupt code is placed here to avoid conflicting with
; the low priority interrupt vector.

HighInt:

;	*** high priority interrupt code goes here ***


		retfie	FAST

;******************************************************************************
;Start of main program
; The main program code is placed here.

Main:

;	*** main code goes here ***
	clrf	INTCON
	clrf	PORTB
	clrf	TRISB
	clrf	PORTC
	clrf	TRISC
	movlw	b'01010101'
	movwf	LATB
	movwf	LATC
loop:
	clrwdt
	movf	LATB,W
	movf	LATC,W
	xorlw	0xff
	movwf	LATB
	movwf	LATC
	goto	loop

;******************************************************************************
;End of program - note that each DB must start on even boundary
		ORG		0x1fe0
		DB		Project
		DB		":", Version
		END
