;**********************************************************************
;                                                                     *
;    Filename:	    DC-DC.asm                                         *
;    Date:          03/02/2012                                        *
;    File Version:  00                                                *
;    Checksum:		0x                                                * 
;                                                                     *
;    Author:        Paul E. Schoen                                    *
;    Company:       P S Technology, Inc.                              *
;                                                                     *
;**********************************************************************
;    Notes:                                                           *
; 03/02/2012 -  - Create from PWMRLY
; 06/07/2012 - 0xdfcb - Working version with problems from inverting gate drivers
; 
;**********************************************************************

	list		p=16F684		; list directive to define processor
	#include	<P16F684.inc>		; processor specific variable definitions
	
	__CONFIG    _CP_OFF & _BOD_ON & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT & _MCLRE_ON
;	__CONFIG    _CP_OFF & _CPD_OFF & _BOD_ON & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT & _MCLRE_OFF & _FCMEN_OFF & _IESO_OFF

#define	BANK1		banksel 0x80	;Select BANK1, STATUS<5> = 1
#define	BANK0		banksel 0x00	;Select BANK0, STATUS<5> = 0


;PWM frequency may be 500, 1000, 2000, 4000, 8000, 16000
#define		PWMFREQ		2000
#define		CLKFREQ		8000

; Actual A/D values for measured inputs

;Bit 7=0 for left justified, bit 6 = 0 for Vdd ref
#define		SEL_AD0		b'00000001'				;<4:2> 000 = AD0
#define		SEL_AD1		b'00000101'				;001 = AD1
#define		SEL_AD2		b'00001001'				;010 = AD2
#define		SEL_AD3		b'00001101'				;011 = AD3
#define		SEL_AD4		b'00010001'				;100 = AD4
#define		SEL_AD5		b'00010101'				;101 = AD5
#define		SEL_AD6		b'00011001'				;110 = AD6
#define		SEL_AD7		b'00011101'				;111 = AD7

; ***** PORTA definitions *****
PGD			equ		RA0		;
PGC			equ		RA1			;
EXTINT		equ		RA2			;
VPP			equ		RA3
A4			equ		RA4
A5			equ		RA5

; ***** PORTC definitions *****
GRNLED		equ		RC0	
REDLED		equ		RC1	
C2			equ		RC2	
C3			equ		RC3
PWMB		equ		RC4
PWMA		equ		RC5

;***** VARIABLE DEFINITIONS *****
;	ORG	0x20
tm0count	EQU	0x20			;Increments 250 uSec or 4 KHz
tm0count1	EQU	0x21			;Increments 64 mSec or 18 Hz
Vadj		EQU	0x22			;Voltage on Input = AN7
PWMvalue	EQU	0x23			;PWM value 0-168 
DelayCtr	EQU	0x24
DelayCount	EQU	0x25			;Delay in 10 mSec increments, 4 = 1 sec
PORTCdata	EQU	0x26

;	ORG	0x71			;Dual address overlapped with 0xf1-0xff
w_temp		EQU	0x71			; variable used for context saving
status_temp	EQU	0x72			; variable used for context saving

;**********************************************************************
	ORG		0x000			; processor reset vector
	nop						;Req'd by debugger?
  	goto	Main			; go to beginning of program


	ORG		0x004			; interrupt vector location
	goto		Isr

;**********************************************************************
	ORG		0x008
Isr:
	movwf		w_temp			; save off current W register contents
	movf		STATUS,w		; move status register into W register
	movwf		status_temp		; save off contents of STATUS register
	bcf			INTCON,GIE		;Turn off Global Interrupts

Timer0InterruptCheck:
	BANK0							;*** Need this if INT occurs in BANK1
	btfsc	INTCON,T0IF				;Bank 0
	goto	Timer0Interrupt			;Yes
	
ADIFInterruptCheck:
	btfsc	PIR1,ADIF				;Bank 0
	goto	A_DConverterInterrupt	;Yes

ExtInterruptCheck:
	btfsc	INTCON,INTF				;Bank 0
	goto	ExtInterrupt	;Yes

Timer0Interrupt
	movlw	d'164'				;Set up for 100 uSec (256-164 = 92), 50 uSec (256-212 = 42)
	movwf	TMR0
	incf	DelayCtr,f			;For time delay
	incf	tm0count,f			;Rolls over 100*256 uSec = 25.6 mSec = 39 Hz
	btfss	STATUS, Z
	goto	EndTimer0Interrupt

	incf	tm0count1,F			;Increments at 25.6 mSec 39 Hz
	bsf		ADCON0, GO			;Bank 0, START Next conversion 39/sec

EndTimer0Interrupt:
	bcf		INTCON,T0IF			;Bank 0, Clear TMR0 Interrupt Flag
	goto	EndIsr

;When A/D converter has a result, store in proper register and select next
A_DConverterInterrupt:			;100 uSec conversion
	bcf		PIR1, ADIF			;Clear Interrupt Flag; Bank 0

;Process AD_Vin = AD7	
	movf	ADRESH,W			;Bank 0
	movwf	Vadj					;0-255 corresponds to 0-30V
	goto	EndIsr

ExtInterrupt:
	bcf		PORTCdata, REDLED		;Indicate error
	bsf		PORTCdata, GRNLED
	movf	PORTCdata, W
	movwf	PORTC
	bcf		INTCON, INTF		;Clear flag

EndIsr:
	movf		status_temp,w	; retrieve copy of STATUS register
	movwf		STATUS			; restore pre-isr STATUS register contents
	swapf		w_temp,f
	swapf		w_temp,w		; restore pre-isr W register contents
	retfie						; return from interrupt, enable GIE

;********************************* Start of Program ********************************
Main
	BANK1
	clrf	ANSEL				;Bank 1, Set all to digital
	bsf		ANSEL,ANS0			;Adjust pot
	movlw	b'01110001'			;Set 8 MHz frequency
	movwf	OSCCON
	movlw	b'00100000'			;fOsc/32 = 8MHz/32 = 4 uSec * 11 = 44 uSec (sim test 50)
	movwf	ADCON1				;Bank 1
	bsf		PIE1,ADIE			;Bank 1, Enable ADC interrupt

	movlw	b'11111111'	;RA<0:5> input
	movwf	TRISA
	movlw	b'00111100'	;RC0:1 output,RC<2:3> input, Output RC<4:5>(PWM) Inputs until set up Outputs
	movwf	TRISC		;

; Timer 0 source internal, prescale 2:1 = 1000 KHz/256 = 3904 Hz				
	movlw	b'10000000'	
	movwf	OPTION_REG	;Bank 1, TIMER0 Prescaler <0:2>, Assigned <3>, Timer 0 Internal <5>, INT falling edge <6>, disable port A pullups <7>

	BANK0

	movlw	b'01101100'
	movwf	PORTA
	clrf	PORTCdata
	bsf		PORTCdata, REDLED
	bcf		PORTCdata, GRNLED
	movf	PORTCdata, W
	movwf	PORTC

;ADCON0 <7> Just, <6> Ref, <5:2> CHS, <1> GO/Done, <0> ADON
	movlw	SEL_AD0		;Left justified, Vdd Vref, Select AN7, ON b'00011101'
	movwf	ADCON0		;Bank 0

;------------------------------ PWM setup --------------------------------------
; PWM 100% determined by (PR2+1) * PRE counts
; PWM period determined by: (PR2+1) * 4 * TMR2prescale / Clock
; PR2=124 => 336 uSec
;Duty Cycle (time) is determined by: CCP * Tosc * TMR2prescale
; 1.00 kHz => 1000 uSec
; Clock is 8 MHz
;At 1 kHz, for CCP(MSB) = 167, tOn = 250*4*4/8 = 500 uSec (50%)
;Duty Cycle = CCPR / (4 * (PR2+1)) = CCPR / 1000 = CCPR(MSB)/250
; Keeping LSBs = 0, an MSB of 168 is 100%, and 17 is about 10%

	movlw	b'10001111'	;<7:6>Half Bridge <5:4>LSB  <3:0>1111: Active LOW
						;1100: Active HIGH output (High turns MOSFET ON)
	movwf	CCP1CON		;Bank 0, Set up compatible PWM on output RC5

#if PWMFREQ == 500

	movlw	b'00000011'		;Prescale 16
	movwf	T2CON

	movlw	0x7d			;Set Duty Cycle 50% 250
	movwf	CCPR1L			;Bank 0, MSBs
	movlw	b'11001111'
	andwf	CCP1CON, f		;Bank 0, Set LSBs to zero
	bsf		CCP1CON, 5
	movlw	b'00000111'
	movwf	PWM1CON
	BANK1

	movlw	d'249'
	movwf	PR2			;Bank 1
#endif

#if PWMFREQ == 1000

	movlw	b'00000011'		;Prescale 16
	movwf	T2CON

	movlw	0x3e			;Set Duty Cycle 50% 3e
	movwf	CCPR1L			;Bank 0, MSBs
	movlw	b'11001111'
	andwf	CCP1CON, f		;Bank 0, Set LSBs to zero
	bsf		CCP1CON, 5
	movlw	b'00000111'
	movwf	PWM1CON
	BANK1

	movlw	d'124'
	movwf	PR2			;Bank 1
#endif

#if PWMFREQ == 2000

	movlw	b'00000011'		;Prescale 16
	movwf	T2CON

	movlw	0x1f			;1f = 31d = Set Duty Cycle 50% 248 uSec
	movwf	CCPR1L			;Bank 0, MSBs
	movlw	b'11001111'
	andwf	CCP1CON, f		;Bank 0, Set LSBs to zero
;	bsf		CCP1CON, 5
	movlw	b'00000111'
	movwf	PWM1CON
	BANK1

	movlw	d'61'		;0.496 = 2.016 kHz
	movwf	PR2			;Bank 1

#endif

#if PWMFREQ == 4000

	movlw	b'00000001'		;Prescale 4
	movwf	T2CON

	movlw	0x3e			;Set Duty Cycle 50% 250
	movwf	CCPR1L			;Bank 0, MSBs
	movlw	b'11001111'
	andwf	CCP1CON, f		;Bank 0, Set LSBs to zero
	bsf		CCP1CON, 5
	movlw	b'00000111'
	movwf	PWM1CON
	BANK1

	movlw	d'124'		;
	movwf	PR2			;Bank 1

#endif

#if PWMFREQ == 8000

	movlw	b'00000001'		;Prescale 4
	movwf	T2CON

	movlw	0x1f			;Set Duty Cycle 50% 250
	movwf	CCPR1L			;Bank 0, MSBs
	movlw	b'11001111'
	andwf	CCP1CON, f		;Bank 0, Set LSBs to zero
;	bsf		CCP1CON, 5
	movlw	b'00000010'
	movwf	PWM1CON
	BANK1

	movlw	d'61'		;
	movwf	PR2			;Bank 1

#endif

#if PWMFREQ == 16000

	movlw	b'00000001'		;Prescale 4
	movwf	T2CON

	movlw	0x0f			;Set Duty Cycle 50% 250
	movwf	CCPR1L			;Bank 0, MSBs
	movlw	b'11001111'
	andwf	CCP1CON, f		;Bank 0, Set LSBs to zero
	bsf		CCP1CON, 5
	movlw	b'00000010'
	movwf	PWM1CON
	BANK1

	movlw	d'30'		;
	movwf	PR2			;Bank 1

#endif
	BANK0
	bsf		PWM1CON, PRSEN	;Enable PWM auto-restart
	movlw	b'11000000'		;ENable external interrupt shutdown
;	movlw	b'10000000'		;Disable external interrupt shutdown
	movwf	ECCPAS			;Bank 0: <7> shutdown, <6:4> 100 = Ext Int shutdown, <3:2> A,C=0, <1:0> B,D=0
	bsf		T2CON,TMR2ON	;Bank 0 - turn timer on

;==============================================================

;======================= Comparator Setup =====================

#ifdef	USE_COMP
;Minimum reference voltage is Vdd/24 = 0.208V
	BANK1
	movlw	b'10100001'		;Reference 0.208V
	movwf	VRCON			;Bank 1
	BANK0
	movlw	b'010'			;<7>C2out, <6>C1out, <5>C2inv, <4>C1inv, <3>CIS, <2:0> CM
	movwf	CMCON0			;Bank 0
#endif
;==============================================================

	BANK0

	bsf		INTCON,PEIE		;Bank 0, Enable peripheral interrupts
	bsf		INTCON,INTE		;Enable external interrupt
	bcf		INTCON,GIE 		;Interrupt on Change DISabled
	bsf		INTCON,T0IE 	;Interrupt on TIMER0 Overflow Enabled
	bcf		INTCON,T0IF 	;Clear TIMER0 Overflow Interrupt Flag

	bcf		PIR1, TMR2IF	;Bank 0

	bsf		ADCON0, GO		;Bank 0, start conversion
	bsf		INTCON,GIE		;Turn on Global Interrupts

	clrf	TMR0			;Bank 0

;Wait for voltages to settle
	movlw	d'10'			;10 * 10 = 100 mSec
	movwf	DelayCount
	call	Delay10

	BANK1
	movlw	b'11001111'
	andwf	TRISC, F		;Bank 1: Clear bits for PWM output
	BANK0
	bcf		ECCPAS, ECCPASE	;Bank 0: Enable PWM

; *********************************** Main Program Loop ****************************
Loop:

	clrwdt
	btfss	ECCPAS, ECCPASE			;1 means shutdown
	goto	ClearError				;0 means PWM is working
	goto	SetError

ClearError:
	movlw	d'50'
	movwf	DelayCount
	call	Delay10
	bsf		PORTCdata, REDLED	
	bcf		PORTCdata, GRNLED
	movf	PORTCdata, W
	movwf	PORTC	
	goto	EndLoop

SetError:
	bcf		PORTCdata, REDLED	
	bsf		PORTCdata, GRNLED
	movf	PORTCdata, W
	movwf	PORTC	
	movlw	d'10'
	movwf	DelayCount
	call	Delay10

EndLoop:
	goto	Loop				;Wait for voltage to come back up


; *********************** Adjust PWM to value in PWMval **********************
; Use MSB only, so 100% is 168
; Duty Cycle ratio is (CCPR1L + CCP1CON<4:5>)/(4*(PR2+1)
; PR2 = 65, Max PWM = 1024/264
; 

SetPWM:
	movf	PWMvalue,W
	movwf	CCPR1L
	bcf		CCP1CON, 5			;Set LSBs to 0; Bank 0
	bcf		CCP1CON, 4
	return

; *********************** Turn PWM Output OFF and Latch ***************
ShutDownPWM:
	bsf		ECCPAS,ECCPASE 			;Shut down PWM
	return

; *********************** Turn PWM Output OFF *************************
DisablePWM:
	bsf		ECCPAS,ECCPASE 			;Shut down PWM
	return

; *********************** Turn PWM Output ON **************************
EnablePWM:
	bcf		ECCPAS,ECCPASE 			;Enable PWM
	return

; ***************** Delay 10 mSec * DelayCount *************************
Delay10:
	movlw	d'155'				;255-155 = 100 counts at 100 uSec = 10 mSec
	movwf	DelayCtr
Delay10a:			;
	clrwdt
	movf	DelayCtr,W
	iorwf	DelayCtr,W
	btfss	STATUS,Z
	goto	Delay10a

	decf	DelayCount, F
	btfss	STATUS,Z
	goto	Delay10

	return						;return when delay = 0

	END                       ; directive 'end of program'

