;**********************************************************************
;                                                                     *
;    Filename:	    SR_motor_3_phase.asm                                         *
;    Date:          08/14/2012                                        *
;    File Version:  00                                                *
;    Checksum:		0x                                                * 
;                                                                     *
;    Author:        Paul E. Schoen                                    *
;    Company:       P S Technology, Inc.                              *
;                                                                     *
;************************************************************************************
;    Notes:                                                           				*
; 08/14/2012 - 0d157 - Create from DC-DC 	* 
;************************************************************************************

	list		p=16F684		; list directive to define processor
	#include	<P16F684.inc>		; processor specific variable definitions
	
	__CONFIG    _CP_OFF & _BOD_ON & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT & _MCLRE_OFF
;	__CONFIG    _CP_OFF & _CPD_OFF & _BOD_ON & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT & _MCLRE_OFF & _FCMEN_OFF & _IESO_OFF

#define	BANK1		banksel 0x80	;Select BANK1, STATUS<5> = 1
#define	BANK0		banksel 0x00	;Select BANK0, STATUS<5> = 0


;PWM frequency may be 500, 1000, 2000, 4000, 8000, 16000
#define		PWMFREQ		2000
#define		CLKFREQ		8000
;2/5/1 good, 1,2,2 ok no start, 3,5,1 good, 3/3/1 good, 3/3/0 good, 3/2/0 ok, 2/6/0 good, 4/6/0 good, 4/3 good
;2/2/0 good,
;6/3/0 good, 5/3/0 OK, 2/2/2 good, 8/8/0 good, 6/9/3 good?, 8/6/2 OK
#define		DELAY1		6	;delay for single drive
#define		DELAY2		6	;delay for double drive
#define		DELAYOFF	6

; Actual A/D values for measured inputs

;Bit 7=0 for left justified, bit 6 = 0 for Vdd ref
#define		SEL_AD0		b'00000001'				;<4:2> 000 = AD0
#define		SEL_AD1		b'00000101'				;001 = AD1
#define		SEL_AD2		b'00001001'				;010 = AD2																																																																																																																																																																																																																																																																																																			
#define		SEL_AD3		b'00001101'				;011 = AD3
#define		SEL_AD4		b'00010001'				;100 = AD4
#define		SEL_AD5		b'00010101'				;101 = AD5
#define		SEL_AD6		b'00011001'				;110 = AD6
#define		SEL_AD7		b'00011101'				;111 = AD7

; ***** PORTA definitions *****
PGD			equ		RA0		;
PGC			equ		RA1			;
SPEED		equ		RA2			;
VPP			equ		RA3
A4			equ		RA4
A5			equ		RA5

; ***** PORTC definitions *****
DRVA		equ		RC0	
DRVB		equ		RC1	
DRVC		equ		RC2	
GRNLED		equ		RC3
PWMB		equ		RC4
PWMA		equ		RC5

;**********Flags**********
START		equ		0			;Set during startup for soft PWM

;***** VARIABLE DEFINITIONS *****
;	ORG	0x20
tm0count	EQU	0x20			;Increments 256 uSec or 3.906 KHz
tm0count1	EQU	0x21			;Increments 64 mSec or 18 Hz
Vspeed		EQU	0x22			;Voltage on Input = AN3
PWMvalue	EQU	0x23			;PWM value 0-168 
DelayCtr	EQU	0x24
DelayCount	EQU	0x25			;Delay in 10 mSec increments, 4 = 1 sec
PORTCdata	EQU	0x26
Flags		equ	0x27


;	ORG	0x71			;Dual address overlapped with 0xf1-0xff
w_temp		EQU	0x71			; variable used for context saving
status_temp	EQU	0x72			; variable used for context saving

;**********************************************************************
	ORG		0x000			; processor reset vector
	nop						;Req'd by debugger?
  	goto	Main			; go to beginning of program


	ORG		0x004			; interrupt vector location
	goto		Isr

;**********************************************************************
	ORG		0x008
Isr:
	movwf		w_temp			; save off current W register contents
	movf		STATUS,w		; move status register into W register
	movwf		status_temp		; save off contents of STATUS register
	bcf			INTCON,GIE		;Turn off Global Interrupts

Timer0InterruptCheck:
	BANK0							;*** Need this if INT occurs in BANK1
	btfsc	INTCON,T0IF				;Bank 0
	goto	Timer0Interrupt			;Yes
	
ADIFInterruptCheck:
	btfsc	PIR1,ADIF				;Bank 0
	goto	A_DConverterInterrupt	;Yes

ExtInterruptCheck:
	btfsc	INTCON,INTF				;Bank 0
	goto	ExtInterrupt	;Yes

Timer0Interrupt						;256 uSec = 3906 Hz
	movlw	d'164'				;Set up for 100 uSec (256-164 = 92), 50 uSec (256-212 = 42)
	movwf	TMR0
	incf	DelayCtr,f			;For time delay
	incf	tm0count,f			;Rolls over 100*256 uSec = 25.6 mSec = 39 Hz
	btfss	STATUS, Z
	goto	Timer0CheckStart

	incf	tm0count1,F			;Increments at 25.6 mSec 39 Hz
	bsf		ADCON0, GO			;Bank 0, START Next conversion 39/sec

Timer0CheckStart:
	btfss	Flags, START
	goto	EndTimer0Interrupt

Timer0Startup
	movf	PORTC,W
	iorlw	b'00110000'
	movwf	PORTC		;Set PWMs high (off)
	movf	tm0count,W
	andlw	0fh
	sublw	01h
	bnz		T0S10

	bcf		PORTC,PWMB			;When 0001
	goto	EndTimer0Interrupt

T0S10:
	movf	tm0count,W
	andlw	0fh
	sublw	09h
	bnz		EndTimer0Interrupt
	bcf		PORTC,PWMA			;When 1001

EndTimer0Interrupt:
	bcf		INTCON,T0IF			;Bank 0, Clear TMR0 Interrupt Flag
	goto	EndIsr

;When A/D converter has a result, store in proper register and select next
A_DConverterInterrupt:			;100 uSec conversion
	bcf		PIR1, ADIF			;Clear Interrupt Flag; Bank 0

;Process AD_Vin = AD7	
	movf	ADRESH,W			;Bank 0
	movwf	Vspeed					;0-255 corresponds to 0-30V
	goto	EndIsr

ExtInterrupt:
	bcf		PORTCdata, GRNLED
	movf	PORTCdata, W
	movwf	PORTC
	bcf		INTCON, INTF		;Clear flag

EndIsr:
	movf		status_temp,w	; retrieve copy of STATUS register
	movwf		STATUS			; restore pre-isr STATUS register contents
	swapf		w_temp,f
	swapf		w_temp,w		; restore pre-isr W register contents
	retfie						; return from interrupt, enable GIE

;********************************* Start of Program ********************************
Main
	BANK1
	clrf	ANSEL				;Bank 1, Set all to digital
	bsf		ANSEL,ANS2			;Adjust pot
	movlw	b'01110001'			;Set 8 MHz frequency
	movwf	OSCCON
	movlw	b'00100000'			;fOsc/32 = 8MHz/32 = 4 uSec * 11 = 44 uSec (sim test 50)
	movwf	ADCON1				;Bank 1
	bsf		PIE1,ADIE			;Bank 1, Enable ADC interrupt

	movlw	b'11111111'	;RA<0:5> input
	movwf	TRISA
	movlw	b'11110000'	;RC0:3 output
	movwf	TRISC		;

	BANK0
	movlw 	07h 		;Set RC<4,1:0> to
	movwf 	CMCON0 		;digital I/O
; Timer 0 source internal, prescale 2:1 = 1000 KHz/256 = 3904 Hz				
	movlw	b'10000000'
	
	BANK1
	movwf	OPTION_REG	;Bank 1, TIMER0 Prescaler <0:2>, Assigned <3>, Timer 0 Internal <5>, INT falling edge <6>, disable port A pullups <7>

	BANK0

	movlw	b'01101100'
	movwf	PORTA
	clrf	PORTCdata
	bsf		PORTCdata, GRNLED
	bcf		PORTCdata, DRVA
	bsf		PORTCdata, DRVB
	bsf		PORTCdata, DRVC
	movf	PORTCdata, W
	movwf	PORTC
	bsf		INTCON,PEIE		;Bank 0, Enable peripheral interrupts
	bcf		INTCON,GIE 		;Interrupt on Change DISabled
	bsf		INTCON,T0IE 	;Interrupt on TIMER0 Overflow Enabled
	bcf		INTCON,T0IF 	;Clear TIMER0 Overflow Interrupt Flag
	clrf	TMR0			;Bank 0
	bsf		INTCON,GIE		;Turn on Global Interrupts

;Perform a software soft start-up for the PWM before turning it on at 50% duty cycle:
;Use TM0 interrupt and turn on RC4 for 1 counts, off for 7 counts, then RC5 on for 1, off for 7
;16 counts at 3.9 kHz is 243 Hz
;00000000: RC4 on
;00000001: RC4 off
;00001000: RC5 on
;00001001: RC5 off
	movlw	0x01
	movwf	Flags		;Set START flag
	movlw	d'10'
	movwf	DelayCount	;Wait 2500 mSec
	call	Delay10

	bcf		Flags,START

;ADCON0 <7> Just, <6> Ref, <5:2> CHS, <1> GO/Done, <0> ADON
	movlw	SEL_AD2		;Left justified, Vdd Vref, Select AN7, ON b'00011101'
	movwf	ADCON0		;Bank 0

	BANK0
	bsf		T2CON,TMR2ON	;Bank 0 - turn timer on

;==============================================================

	bcf		PIR1, TMR2IF	;Bank 0

	bsf		ADCON0, GO		;Bank 0, start conversion

;Wait for voltages to settle
	movlw	d'10'			;10 * 10 = 100 mSec
	movwf	DelayCount
	call	Delay10

; *********************************** Main Program Loop ****************************
Loop:

	clrwdt

	clrf	PORTCdata
	bsf		PORTCdata, GRNLED
	bsf		PORTCdata, DRVA
;	bsf		PORTCdata, DRVB
	movf	PORTCdata, W
	movwf	PORTC	
	movlw	DELAY1
	movwf	DelayCount
	call	Delay10

	clrf	PORTCdata
	bcf		PORTCdata, GRNLED
	movf	PORTCdata, W
	movwf	PORTC	
	movlw	DELAYOFF
	movwf	DelayCount
	call	Delay10

	clrf	PORTCdata
	bcf		PORTCdata, GRNLED
	bsf		PORTCdata, DRVA
	bsf		PORTCdata, DRVB
	movf	PORTCdata, W
	movwf	PORTC	
	movlw	DELAY2
	movwf	DelayCount
	call	Delay10

	clrf	PORTCdata
	bcf		PORTCdata, GRNLED
	movf	PORTCdata, W
	movwf	PORTC	
	movlw	DELAYOFF
	movwf	DelayCount
	call	Delay10

	clrf	PORTCdata
	bsf		PORTCdata, GRNLED
	bsf		PORTCdata, DRVB
;	bsf		PORTCdata, DRVC
	movf	PORTCdata, W
	movwf	PORTC	
	movlw	DELAY1
	movwf	DelayCount
	call	Delay10

	clrf	PORTCdata
	bcf		PORTCdata, GRNLED
	movf	PORTCdata, W
	movwf	PORTC	
	movlw	DELAYOFF
	movwf	DelayCount
	call	Delay10

;	goto	Loop

	clrf	PORTCdata
	bcf		PORTCdata, GRNLED
	bsf		PORTCdata, DRVC
	bsf		PORTCdata, DRVB
	movf	PORTCdata, W
	movwf	PORTC	
	movlw	DELAY2
	movwf	DelayCount
	call	Delay10

	clrf	PORTCdata
	bcf		PORTCdata, GRNLED
	movf	PORTCdata, W
	movwf	PORTC	
	movlw	DELAYOFF
	movwf	DelayCount
	call	Delay10

	clrf	PORTCdata
	bsf		PORTCdata, GRNLED
	bsf		PORTCdata, DRVC
;	bsf		PORTCdata, DRVB
	movf	PORTCdata, W
	movwf	PORTC	
	movlw	DELAY1
	movwf	DelayCount
	call	Delay10

	clrf	PORTCdata
	bcf		PORTCdata, GRNLED
	movf	PORTCdata, W
	movwf	PORTC	
	movlw	DELAYOFF
	movwf	DelayCount
	call	Delay10

	clrf	PORTCdata
	bcf		PORTCdata, GRNLED
	bsf		PORTCdata, DRVA
	bsf		PORTCdata, DRVC
	movf	PORTCdata, W
	movwf	PORTC	
	movlw	DELAY2
	movwf	DelayCount
	call	Delay10

	clrf	PORTCdata
	bcf		PORTCdata, GRNLED
	movf	PORTCdata, W
	movwf	PORTC	
	movlw	DELAYOFF
	movwf	DelayCount
	call	Delay10

EndLoop:
	goto	Loop


; ***************** Delay 10 mSec * DelayCount *************************
Delay10:
	movf	DelayCount,W
	iorwf	DelayCount,W
	btfsc	STATUS,Z
	goto	DelayExit

	movlw	d'155'				;255-155 = 100 counts at 100 uSec = 10 mSec
	movwf	DelayCtr
Delay10a:			;
	clrwdt
	movf	DelayCtr,W
	iorwf	DelayCtr,W
	btfss	STATUS,Z
	goto	Delay10a

	decf	DelayCount, F
	btfss	STATUS,Z
	goto	Delay10
DelayExit:
	return						;return when delay = 0

	END                       ; directive 'end of program'

